/*****************************************************************************
*
*   Name:   I2C Character LCD Library for use with PCF8574A Port Expander
*
*   Date:   2008.02.24
*   Ver:    1.00
*   Who:    GJO
*
*   History:
*
*   Ver     Date        Who     Description
*   ---------------------------------------------------------------------------
*   1.00    2008.02.24  GJO     Initial Version
*
*
*   HARDWARE INFORMATION:
*   =====================
*
*   LCD used:   LUMEX LCM-S01602DSF/A
*
*   LCD     LCD         IO PINS
*   PIN     FUNCTION    PCF8574A        Comments
*--------------------------------------------------------------------------
*   1       VSS                         GND
*   2       VCC                         +5V
*   3       VO                          10K POT VCC to GND  Contrast Voltage
*   4       RS          P4
*   5       WR                          GND - Busy Detection not used
*   6       E           P7
*   7-10    DB0-3                       Not used - 4-bit mode used
*   11      DB4         P0
*   12      DB5         P1
*   13      DB6         P2
*   14      DB7         P3
*
******************************************************************************/

#include "PCF8574A_Driver.c"

char const addLUT[2] = {0x80,0xC0};  // RAM Offset for 2x16 LCD only
unsigned char _lcd_addr, _lcd_line;

// LCD functions
void wr_half(unsigned char);
void wr_disp(unsigned char, int);
void init_display(void);
void line(char);
void clear_display(void);
void set_LCD_cur(char, char );
void put_I2C_LCD(unsigned char );

#define LCD_CMD  0b00000000
#define LCD_DATA 0b00010000
#define E_HIGH   0b10000000
#define E_LOW    0b01111111

/******************************************************************************
* Function:     wr_half(unsigned char data)
* Description:  Write a nibble to the LCD display using DB4-DB7
* Returns:      none
*******************************************************************************/
void wr_half(unsigned char data)
{
    set_output_PCF8574A(data | E_HIGH);
    set_output_PCF8574A(data & E_LOW);
    delay_ms(3);    // allow display time to accept the new data
}

/******************************************************************************
* Function:     wr_disp(unsigned char data)
* Description:  Write a byte to the LCD display
* Returns:      none
*******************************************************************************/
void wr_disp(unsigned char data, int cmd_mode)
{
    int temp;

    // Write to display upper nibble
    temp = data >> 4;
    temp = temp | cmd_mode;
    set_output_PCF8574A(temp | E_HIGH);
    set_output_PCF8574A(temp & E_LOW);

    // Write to display lower nibble
    temp = data & 0x0f;
    temp = temp | cmd_mode;
    set_output_PCF8574A(temp | E_HIGH);
    set_output_PCF8574A(temp & E_LOW);
}



/******************************************************************************
* Function:     init_disp(void)
* Description:  Setup LCD display for application
* Returns:      none
*******************************************************************************/
void init_display(void)
{
    // always command mode used here
    wr_half(0x03);          // Send 0x33 and 0x32 to enter 4 bit mode
    wr_half(0x03);          // See HD44780U.PDF datasheet for all
    wr_half(0x03);          // configuration options
    wr_half(0x02);

    wr_disp(0x28, LCD_CMD);  // Mode: 4-Bit, 2-line, 5x7
    wr_disp(0x01, LCD_CMD);  // Clear Display memory
    delay_ms(20);
    wr_disp(0x10, LCD_CMD);  // Set cursor to move
    wr_disp(0x06, LCD_CMD);  // set cursor to move right, no shifting of LCD display
    wr_disp(0x0c, LCD_CMD);  // turn display on, cursor off, cursor blinking off
}

/******************************************************************************
* Function:     void line(char which_line)
* Description:  Move to line specified by which_line
* Uses:         which_line to selct LCD line to be used
* Returns:      none
*******************************************************************************/
void line(char which_line)
{
    _lcd_addr = addLUT[which_line - 1];
    wr_disp(_lcd_addr, LCD_CMD);                     // move to which_line line
    _lcd_line = which_line;
}

/******************************************************************************
* Function:     void clear_display(void)
* Description:  Clear LCD
* Uses:         none
* Returns:      none
*******************************************************************************/
void clear_display(void)
{
    wr_disp(0x01, LCD_CMD);              // Clear display and home cursor
    delay_ms(20);
    line(1);
}

/******************************************************************************
* Function:     void set_LCD_cur(char LCDrow, char LCDcol)
* Description:  Set row, col cursor position in LCD
* Uses:         LCDrow, LCDcol
* Returns:      none
*******************************************************************************/
void set_LCD_cur(char LCDrow, char LCDcol)
{
    _lcd_addr = addLUT[LCDrow] + LCDcol;
    wr_disp(_lcd_addr, LCD_CMD);          // move to row and column
    _lcd_line = LCDrow;
}



/******************************************************************************
* Function:     void disp_char(unsigned char c)
* Description:  display a character on LCD
* Uses:         character c, gl _lcd_line, _lcd_addr
* Returns:      none
*******************************************************************************/
void put_I2C_LCD(unsigned char c)
{
    wr_disp(c, LCD_DATA);
    _lcd_addr++;

    if(_lcd_addr == 16)
        line(2);

    if(_lcd_addr == 32)
        line(1);
}